package com.wcs.back.controller;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.wcs.back.domain.WcsResult;
import com.wcs.back.domain.WcsResult2;
import com.wcs.back.service.IWcsService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

/**
 * wcs回调Controller
 *
 * @author zf
 * @date 2022-11-11
 */
@RestController
@RequestMapping("/wms")
@Api(value = "wcs回调控制器", tags = {"wcs回调管理"})
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class WcsController {

    private final IWcsService wcsService;

    /**
     * 普通搬运任务下发
     */
    @ApiOperation("普通搬运任务下发")
    @PostMapping("/SetUp_Task")
    public WcsResult SetUp_Task(@RequestBody String json, HttpServletRequest request) throws JsonProcessingException {
        WcsResult wcsResult = wcsService.SetUp_Task(json, request);
        return wcsResult;
    }

    /**
     * 机械臂拆托任务下发
     */
    @ApiOperation("机械臂拆托任务下发")
    @PostMapping("/Dismantling_Task")
    public WcsResult Dismantling_Task(@RequestBody String json, HttpServletRequest request) throws JsonProcessingException {
        WcsResult wcsResult = wcsService.Dismantling_Task(json, request);
        return wcsResult;
    }

    /**
     * CTU任务下发
     */
    @ApiOperation("CTU任务下发")
    @PostMapping("/SetUp_CTUTask")
    public WcsResult SetUp_CTUTask(@RequestBody String json, HttpServletRequest request) throws JsonProcessingException {
        WcsResult wcsResult = wcsService.SetUp_CTUTask(json, request);
        return wcsResult;
    }

    /**
     * 滚筒任务下发
     */
    @ApiOperation("滚筒任务下发")
    @PostMapping("/DrumLine_Task")
    public WcsResult DrumLine_Task(@RequestBody String json, HttpServletRequest request) throws JsonProcessingException {
        WcsResult wcsResult = wcsService.DrumLine_Task(json, request);
        return wcsResult;
    }

    /**
     * 指令下发
     */
    @ApiOperation("指令下发")
    @PostMapping("/Directives")
    public WcsResult Directives(@RequestBody String json, HttpServletRequest request) throws JsonProcessingException {
        WcsResult wcsResult = wcsService.Directives(json, request);
        return wcsResult;
    }

    /**
     * 称重
     */
    @ApiOperation("称重")
    @GetMapping("/GetWeight")
    public WcsResult2 GetWeight(String rollerID) throws JsonProcessingException {
        WcsResult2 wcsResult = wcsService.GetWeight(rollerID);
        return wcsResult;
    }

    /**
     * 获取滚筒线信息
     */
    @ApiOperation("获取滚筒线信息")
    @GetMapping("/GetRollerInfo")
    public WcsResult2 GetRollerInfo() throws JsonProcessingException {
        WcsResult2 wcsResult = wcsService.GetRollerInfo();
        return wcsResult;
    }
}
